@@ -6,9 +6,8 @@
 #include "meta.h"
 #include <bitset>
 
-class hostdata;
-
 static const int RESERVED_DEFVAL = -4223;
+#define ACNG_DEF_PORT "3142"
 
 #define ACFG_REDIRMAX_DEFAULT 5
 
@@ -26,13 +25,15 @@ namespace acfg
 {
 
 extern mstring cachedir, logdir, confdir, fifopath, user, group, pidfile, suppdir,
-reportpage, vfilepat, pfilepat, wfilepat, agentname, remoteport, adminauth, bindaddr, port, sUmask,
-tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx;
+reportpage, vfilepat, pfilepat, wfilepat, agentname, adminauth, bindaddr, port, sUmask,
+tmpDontcacherq, tmpDontcachetgt, tmpDontcache, mirrorsrcs, requestapx,
+cafile, capath;
 
 extern int debug, numcores, offlinemode, foreground, verbose, stupidfs, forcemanaged, keepnver,
 verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize, usewrap,
 exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap, dirperms, fileperms,
-maxtempdelay, redirmax, vrangeops, stucksecs, persistoutgoing, pipelinelen, exsupcount;
+maxtempdelay, redirmax, vrangeops, stucksecs, persistoutgoing, pipelinelen, exsupcount,
+optproxytimeout;
 
 // processed config settings
 extern tHttpUrl proxy_info;
@@ -41,7 +42,7 @@ extern mstring agentheader;
 extern int conprotos[2];
 
 bool SetOption(const mstring &line, bool bQuiet=false, NoCaseStringMap *pDupeChecker=NULL);
-void ReadConfigDirectory(const char*);
+void ReadConfigDirectory(const char*, bool bTestMode);
 
 //! Prepare various things resulting from variable combinations, etc.
 void PostProcConfig();
@@ -81,7 +82,8 @@ extern MYSTD::bitset<TCP_PORT_MAX> *pUserPorts;
 
 extern mstring cacheDirSlash; // guaranteed to have a trailing path separator
 
-mstring EncodeBase64Auth(const cmstring & s);
+void printVar(cmstring &varname);
+//bool ReadOneConfFile(cmstring&);
 } // namespace acfg
 
 namespace rechecks
