@@ -36,6 +36,7 @@ const static tStrPos stmiss(cmstring::npos);
 typedef unsigned int UINT;
 typedef unsigned short USHORT;
 typedef unsigned char UCHAR;
+typedef const char * LPCSTR;
 
 #define citer const_iterator
 
@@ -43,7 +44,7 @@ typedef unsigned char UCHAR;
 #define SZPATHSEPUNIX "/"
 #define CPATHSEPWIN '\\'
 #define SZPATHSEPWIN "\\"
-extern cmstring sPathSep, sPathSepUnix;
+extern cmstring sPathSep, sPathSepUnix, sDefPortHTTP, sDefPortHTTPS;
 
 #ifdef WINDOWS
 #define WIN32
@@ -87,20 +88,20 @@ int getUUID();
 
 typedef MYMAP<mstring, mstring> tStrMap;
 
-inline void trimFront(mstring &s, const char *junk=SPACECHARS)
+inline void trimFront(mstring &s, LPCSTR junk=SPACECHARS)
 {
 	mstring::size_type pos = s.find_first_not_of(junk);
 	if(pos != 0)
 		s.erase(0, pos);
 }
 
-inline void trimBack(mstring &s, const char *junk=SPACECHARS)
+inline void trimBack(mstring &s, LPCSTR junk=SPACECHARS)
 {
 	mstring::size_type pos = s.find_last_not_of(junk);
 	s.erase(pos+1);
 }
 
-inline void trimString(mstring &s, const char *junk=SPACECHARS)
+inline void trimString(mstring &s, LPCSTR junk=SPACECHARS)
 {
 	trimBack(s, junk);
 	trimFront(s, junk);
@@ -134,10 +135,10 @@ tStrPos findHostStart(const mstring & sUri);
 //extern mstring sPathSep, sPathSepUnix, sCR, sCRLF;
 
 // there is memchr and strpbrk but nothing like the last one acting on specified RAW memory range
-static inline const char * mempbrk (const char * membuf, char const * const needles, size_t len)
+static inline LPCSTR  mempbrk (LPCSTR  membuf, char const * const needles, size_t len)
 {
-   for(const char *pWhere=membuf ; pWhere<membuf+len ; pWhere++)
-      for(const char *pWhat=needles; *pWhat ; pWhat++)
+   for(LPCSTR pWhere=membuf ; pWhere<membuf+len ; pWhere++)
+      for(LPCSTR pWhat=needles; *pWhat ; pWhat++)
          if(*pWhat==*pWhere)
             return pWhere;
    return NULL;
@@ -150,7 +151,7 @@ typedef MYSTD::vector<mstring>::iterator tStrVecIter;
 typedef MYSTD::vector<mstring>::const_iterator tStrVecIterConst;
 
 // Sometimes I miss Perl...
-tStrVec::size_type Tokenize(const mstring &in, const char *sep, tStrVec & out, bool bAppend=false, mstring::size_type nStartOffset=0);
+tStrVec::size_type Tokenize(const mstring &in, LPCSTR sep, tStrVec & out, bool bAppend=false, mstring::size_type nStartOffset=0);
 void Join(mstring &out, const mstring & sep, const tStrVec & tokens);
 
 // TODO: __attribute__((externally_visible))
@@ -162,33 +163,59 @@ class tHttpUrl
 {
 public:
 	bool SetHttpUrl(cmstring &uri);
-	/*
-	/// The specified url has a path which is a prefix of the one in *this 
-	UINT HasPathPrefix(const tHttpUrl &another);
-	*/
 	mstring ToURI(bool bEscaped) const;
-	mstring sHost, sPort, sPath;
+	mstring sHost, sPath, sUserPass;
+	bool bIsTransferlEncoded;
+#ifdef HAVE_SSL
+	bool bSSL;
+#endif
+
 	tHttpUrl & operator=(const tHttpUrl &a) 
 	{
-		sHost=a.sHost; sPort=a.sPort; sPath=a.sPath;
+		sHost=a.sHost; sPort=a.sPort; sPath=a.sPath; sUserPass=a.sUserPass;
+#ifdef HAVE_SSL
+		bSSL=a.bSSL;
+#endif
+		bIsTransferlEncoded=a.bIsTransferlEncoded;
 		return *this;
 	};
 	bool operator==(const tHttpUrl &a) const
 	{
-		return a.sHost==sHost && a.sPort == sPort && a.sPath == sPath;
+		return a.sHost==sHost && a.sPort == sPort
+				&& a.sPath == sPath && a.sUserPass == sUserPass
+#ifdef HAVE_SSL
+				&& a.bSSL == bSSL
+#endif
+				&& a.bIsTransferlEncoded == bIsTransferlEncoded
+				;
 	};
 	bool operator!=(const tHttpUrl &a) const
 	{
 		return ! (a==*this);
 	}
-	inline void clear() { sHost=sPort=sPath=""; }
-	
-	/*tHttpUrl(const mstring & host, const mstring & path)
-	: sHost(host), sPath(path)
-	{}*/
-	tHttpUrl()
+	inline void clear() { sHost.clear(); sPort.clear(); sPath.clear();
+	sUserPass.clear();
+#ifdef HAVE_SSL
+	bSSL=false;
+#endif
+	bIsTransferlEncoded=false;
+	}
+	inline tHttpUrl()
+	: bIsTransferlEncoded(false)
+#ifdef HAVE_SSL
+	, bSSL(false)
+#endif
 	{};
-	
+
+	inline cmstring& GetPort() const { return !sPort.empty() ? sPort :
+#ifdef HAVE_SSL
+			(bSSL ? sDefPortHTTPS : sDefPortHTTP);
+#else
+	sDefPortHTTP;
+#endif
+	}
+private:
+	mstring sPort;
 };
 
 #define POKE(x) for(;;) { ssize_t n=write(x, "", 1); if(n>0 || (EAGAIN!=errno && EINTR!=errno)) break;  }
@@ -199,10 +226,10 @@ public:
 void appendLong(mstring &s, long val);
 
 mstring BytesToHexString(const uint8_t b[], unsigned short binLength);
-bool CsAsciiToBin(const char *a, uint8_t b[], unsigned short binLength);
+bool CsAsciiToBin(LPCSTR a, uint8_t b[], unsigned short binLength);
 
 typedef const unsigned char CUCHAR;
-bool CsEqual(const char *a, uint8_t b[], unsigned short binLength);
+bool CsEqual(LPCSTR a, uint8_t b[], unsigned short binLength);
 void StrSubst(mstring &contents, const mstring &from, const mstring &to);
 
 #if SIZEOF_LONG == 8
@@ -230,7 +257,7 @@ void StrSubst(mstring &contents, const mstring &from, const mstring &to);
 #endif
 
 // let the compiler optimize and keep best variant
-inline off_t atoofft(const char *p)
+inline off_t atoofft(LPCSTR p)
 {
 	if(sizeof(long long) == sizeof(off_t))
 		return atoll(p);
@@ -239,7 +266,7 @@ inline off_t atoofft(const char *p)
 	return atol(p);
 }
 
-inline off_t atoofft(const char *p, off_t nDefVal)
+inline off_t atoofft(LPCSTR p, off_t nDefVal)
 {
 	return p ? atoofft(p) : nDefVal;
 }
@@ -290,7 +317,7 @@ inline mstring ltos(long n)
 
 inline mstring offttosH(off_t n)
 {
-	const char * pref[]={"", " KiB", " MiB", " GiB", " TiB", " PiB", " EiB"};
+	LPCSTR  pref[]={"", " KiB", " MiB", " GiB", " TiB", " PiB", " EiB"};
 	for(UINT i=0;i<_countof(pref)-1; i++)
 	{
 		if(n<1024)
@@ -303,10 +330,10 @@ inline mstring offttosH(off_t n)
 	return "INF";
 }
 
-inline void replaceChars(mstring &s, const char *szBadChars, char goodChar)
+inline void replaceChars(mstring &s, LPCSTR szBadChars, char goodChar)
 {
 	for(mstring::iterator p=s.begin();p!=s.end();p++)
-		for(const char *b=szBadChars;*b;b++)
+		for(LPCSTR b=szBadChars;*b;b++)
 			if(*b==*p)
 			{
 				*p=goodChar;
@@ -321,10 +348,10 @@ class tSplitWalk
 {
 	cmstring &s;
 	mstring::size_type start, len, oob;
-	const char *m_seps;
+	LPCSTR m_seps;
 
 public:
-	inline tSplitWalk(cmstring *line, const char *separators=SPACECHARS)
+	inline tSplitWalk(cmstring *line, LPCSTR separators=SPACECHARS)
 	: s(*line), start(0), len(stmiss), oob(line->size()), m_seps(separators) {}
 	inline bool Next()
 	{
@@ -355,7 +382,7 @@ public:
 	}
 	inline mstring str(){ return s.substr(start, len); }
 	inline operator mstring() { return str(); }
-//	inline const char *rest() { return s.c_str() + start; }
+//	inline LPCSTR rest() { return s.c_str() + start; }
 };
 
 //bool CreateDetachedThread(void *(*threadfunc)(void *));
@@ -456,5 +483,12 @@ struct extended_bool
 
 void DelTree(cmstring &what);
 
+struct errnoFmter: public mstring
+{
+	errnoFmter(LPCSTR prefix = NULL);
+};
+
+mstring EncodeBase64Auth(cmstring & s);
+
 #endif // _META_H
 
