@@ -23,7 +23,10 @@ using namespace MYSTD;
 #define _iterPos(it, start) (it-start.begin())/sizeof(it)
 #define sProblemLoc szPath<< ':'<< _iterPos(it, lines)
 
-#define BARF(x) { cerr << x << endl; exit(EXIT_FAILURE); }
+// hint to use the main configuration excluding the complex directives
+bool g_testMode=false;
+
+#define BARF(x) {if(!g_testMode){ cerr << x << endl; exit(EXIT_FAILURE); }}
 #define BADSTUFF_PATTERN "\\.\\.($|%|/)"
 mstring PTHOSTS_PATTERN;
 
@@ -84,6 +87,8 @@ MapNameToString n2sTbl[] = {
 		,{ "PrecacheFor", &mirrorsrcs, 0}
 		,{ "RequestAppendix", &requestapx, 0}
 		,{ "PassThroughPattern", &PTHOSTS_PATTERN, 0}
+		,{ "CApath", &capath, 0}
+		,{ "CAfile", &cafile, 0}
 };
 
 MapNameToInt n2iTbl[] = {
@@ -117,6 +122,7 @@ MapNameToInt n2iTbl[] = {
 		,{ "ReuseConnections", &persistoutgoing, NULL, 10}
 		,{ "PipelineDepth", &pipelinelen, NULL, 10}
 		,{ "ExSuppressAdminNotification", &exsupcount, NULL, 10}
+		,{ "OptProxyTimeout", &optproxytimeout, NULL, 10}
 
 		,{ "DirPerms", &dirperms, NULL, 8}
 		,{ "FilePerms", &fileperms, NULL, 8}
@@ -125,14 +131,16 @@ MapNameToInt n2iTbl[] = {
 void ReadRewriteFile(const string & sFile, const string & sRepName);
 void ReadBackendsFile(const string & sFile, const string &sRepName);
 
-
-
 struct fct_lt_host
 {
-  bool operator()(const tHttpUrl &a, const tHttpUrl &b) const
-  {
-    return strcasecmp(a.sHost.c_str(), b.sHost.c_str()) < 0;
-  }
+	bool operator()(const tHttpUrl &a, const tHttpUrl &b) const
+	{
+		cmstring& ap = a.GetPort();
+		cmstring& bp = b.GetPort();
+		if (ap != bp)
+			return ap < bp;
+		return strcasecmp(a.sHost.c_str(), b.sHost.c_str()) < 0;
+	}
 };
 typedef multimap<tHttpUrl,const string*, fct_lt_host> tMapUrl2StringPtr; 
 typedef tMapUrl2StringPtr::iterator tUrl2RepIter;
@@ -169,6 +177,15 @@ int * GetIntPtr(const string &key, int &base) {
 	return NULL;
 }
 
+void printVar(cmstring &key)
+{
+	string *ps=GetStringPtr(key);
+	if(ps)
+		cout << *ps << endl;
+	int base, *pn=GetIntPtr(key, base);
+	if(pn)
+		cout << *pn <<endl;
+}
 
 // shortcut for frequently needed code, opens the config file, reads step-by-step
 // and skips comment and empty lines
@@ -213,7 +230,7 @@ inline void _FixPostPreSlashes(string &val)
 		val.insert(0, "/", 1);
 }
 
-inline bool _ReadMainConfiguration(const string & szFilename)
+bool ReadOneConfFile(const string & szFilename)
 {
 	tCfgIter itor(szFilename);
 	itor.reader.CheckGoodState(true);
@@ -230,7 +247,7 @@ inline bool _ReadMainConfiguration(const string & szFilename)
 		if(stmiss != pos)
 			itor.sLine.erase(pos);
 
-		if(! SetOption(itor.sLine, false, &dupeCheck))
+		if(! SetOption(itor.sLine, g_testMode, &dupeCheck))
 			BARF("Error reading main options, terminating.");
 	}
 	return true;
@@ -365,9 +382,6 @@ inline void AddRemapInfo(bool bAsBackend, const string & token,
 		if(! url.SetHttpUrl(token))
 			BARF(token + " <-- bad URL detected");
 		_FixPostPreSlashes(url.sPath);
-		
-		if(url.sPort.empty())
-			url.sPort = remoteport;
 
 		if (bAsBackend)
 			repoparms[repname].m_backends.push_back(url);
@@ -458,58 +472,6 @@ void _AddHooksFile(cmstring& vname)
 	repoparms[vname].m_pHooks = &hs;
 }
 
-string EncodeBase64Auth(const string & s)
-{
-	int cols=0, bits=0, c=0, char_count=0;
-	char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-	tStrPos pos=0;
-	string out;
-	while ( pos<s.size() )
-	{
-		c=s[pos++];
-		if('%' == c && pos<s.length()-1
-				&& hexmap[(unsigned)s[pos]]<=15
-				&& hexmap[(unsigned)s[pos+1]]<=15)
-		{
-			c=char(16*hexmap[(unsigned)s[pos]]+hexmap[(unsigned)s[pos+1]]);
-			pos+=2;
-		}
-		bits += c;
-		char_count++;
-		if (char_count == 3)
-		{
-			out+=(alphabet[bits >> 18]);
-			out+=(alphabet[(bits >> 12) & 0x3f]);
-			out+=(alphabet[(bits >> 6) & 0x3f]);
-			out+=(alphabet[bits & 0x3f]);
-			cols += 4;
-			bits = 0;
-			char_count = 0;
-		}
-		else
-		{
-			bits <<= 8;
-		}
-	}
-	if (char_count != 0)
-	{
-		bits <<= 16 - (8 * char_count);
-		out+=(alphabet[bits >> 18]);
-		out+=(alphabet[(bits >> 12) & 0x3f]);
-		if (char_count == 1)
-		{
-			out+=('=');
-			out+=('=');
-		}
-		else
-		{
-			out+=(alphabet[(bits >> 6) & 0x3f]);
-			out+=('=');
-		}
-	}
-	return out;
-}
-
 inline void _ParseLocalDirs(cmstring &value)
 {
 	for(tSplitWalk splitter(&value, ";"); splitter.Next(); )
@@ -664,47 +626,20 @@ bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 
 	else if(0==strcasecmp(key.c_str(), "Proxy"))
 	{
-		if (!proxy_info.SetHttpUrl(value))
-		{
-			cerr << "Invalid proxy specification, aborting..." << endl;
-			exit(EXIT_FAILURE);
-		}
-		tStrPos pos = proxy_info.sHost.rfind('@');
-		if (stmiss != pos)
-		{
-			tStrPos pwStart= startsWithSz(value, "http://") ? 7 : 0;
-			proxy_info.sPath = string("Proxy-Authorization: Basic ")
-					+ EncodeBase64Auth(value.substr(pwStart, pos)) + "\r\n";
-			proxy_info.sHost.erase(0, pos + 1);
-		}
+		if(value.empty())
+			proxy_info=tHttpUrl();
 		else
-			proxy_info.sPath.clear(); // no auth at all, stop confusion
-
-		if (proxy_info.sHost.empty())
 		{
-			cerr << "Invalid proxy specification, aborting..." << endl;
-			exit(EXIT_FAILURE);
-		}
-		if (proxy_info.sPort.empty())
-		{
-			if (proxy_info.sPath.empty()) // guess unless there is any risk...
-			{
-				cerr << "Warning, unknown proxy port, assuming 80." << endl;
-				proxy_info.sPort = "80";
-			}
-			else
-			{
-				cerr << "Error, unknown proxy port!" << endl;
-				exit(EXIT_FAILURE);
-			}
+			if (!proxy_info.SetHttpUrl(value) || proxy_info.sHost.empty())
+				BARF("Invalid proxy specification, aborting...");
 		}
 	}
-	else if(0==strcasecmp(key.c_str(), "LocalDirs"))
+	else if(0==strcasecmp(key.c_str(), "LocalDirs") && !g_testMode)
 	{
 		_ParseLocalDirs(value);
 		return !localdirs.empty();
 	}
-	else if(0==strncasecmp(key.c_str(), "Remap-", 6))
+	else if(0==strncasecmp(key.c_str(), "Remap-", 6) && !g_testMode)
 	{
 		string vname=key.substr(6, key.npos);
 		tStrVec tokens;
@@ -745,10 +680,7 @@ bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 			char *p(0);
 			unsigned long n=strtoul(start, &p, 10);
 			if(n>=TCP_PORT_MAX || !p || '\0' != *p || p == start)
-			{
-				cerr << "Bad port in AllowUserPorts: " << start <<endl;
-				exit(EXIT_FAILURE);
-			}
+				BARF("Bad port in AllowUserPorts: " << start);
 			if(n == 0)
 			{
 				pUserPorts->set();
@@ -767,20 +699,14 @@ bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 				break;
 
 			if (p >= conprotos + _countof(conprotos))
-			{
-				cerr << "Too many protocols specified: " << val << endl;
-				exit(EXIT_FAILURE);
-			}
+				BARF("Too many protocols specified: " << val);
 
 			if (val == "v6")
 				*p = PF_INET6;
 			else if (val == "v4")
 				*p = PF_INET;
 			else
-			{
-				cerr << "IP protocol not supported: " << val << endl;
-				exit(EXIT_FAILURE);
-			}
+				BARF("IP protocol not supported: " << val);
 		}
 	}
 	else
@@ -794,7 +720,7 @@ bool SetOption(const string &sLine, bool bQuiet, NoCaseStringMap *pDupeCheck)
 
 
 //const string * GetVnameForUrl(string path, string::size_type * nMatchLen)
-const string * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathResidual)
+cmstring * GetRepNameAndPathResidual(const tHttpUrl & in, string & sRetPathResidual)
 {
 	sRetPathResidual.clear();
 	
@@ -846,12 +772,12 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 
 	tCfgIter itor(sFile);
 	if(debug&6)
-		cout << "Reading backend file: " << sFile <<endl;
+		cerr << "Reading backend file: " << sFile <<endl;
 
 	if(!itor)
 	{
 		if(debug&6)
-			cout << "No backend data found, file ignored."<<endl;
+			cerr << "No backend data found, file ignored."<<endl;
 		goto try_read_default;
 	}
 	
@@ -859,23 +785,16 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 	{
 		if(debug & LOG_DEBUG)
 			cerr << "backends, got line: " << itor.sLine <<endl;
-		
-		// either a URL line or the termination line and hostname/path have been set before
 
-		if( (startsWithSz(itor.sLine, "http://") && entry.SetHttpUrl(itor.sLine) )
-				||
-			( itor.sLine.empty() && ! entry.sHost.empty() && ! entry.sPath.empty()
-			)
-		)
+		trimBack(itor.sLine);
+
+		if( entry.SetHttpUrl(itor.sLine)
+				||	( itor.sLine.empty() && ! entry.sHost.empty() && ! entry.sPath.empty()) )
 		{
 			_FixPostPreSlashes(entry.sPath);
 #ifdef DEBUG
 			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI(false) <<endl;
 #endif		
-
-			if(entry.sPort.empty())
-				entry.sPort = remoteport;
-
 			repoparms[sRepName].m_backends.push_back(entry);
 			nAddCount++;
 			entry.clear();
@@ -905,9 +824,8 @@ void ReadBackendsFile(const string & sFile, const string &sRepName)
 		}
 		else
 		{
-			cerr << "Bad backend description, around line " << sFile << ":"
-					<< itor.reader.GetCurrentLine() << endl;
-			exit(2);
+			BARF("Bad backend description, around line " << sFile << ":"
+					<< itor.reader.GetCurrentLine());
 		}
 	}
 
@@ -915,7 +833,7 @@ try_read_default:
 	if(nAddCount || endsWithSzAr(sFile, ".default"))
 		return;
 	if(debug>4)
-		cout << "Trying to read replacement from " << sFile << ".default" <<endl;
+		cerr << "Trying to read replacement from " << sFile << ".default" <<endl;
 	ReadBackendsFile(sFile+".default", sRepName);
 
 }
@@ -931,7 +849,7 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 
 	filereader reader;
 	if(debug>4)
-		cout << "Reading rewrite file: " << sFile <<endl;
+		cerr << "Reading rewrite file: " << sFile <<endl;
 	reader.OpenFile(sFile);
 	reader.CheckGoodState(true);
 	reader.AddEofLines();
@@ -944,29 +862,25 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 	{
 		trimFront(sLine);
 
-		if (0==sLine.compare(0, 7, "http://"))
-		{ // TODO: The check above is optional since SetHttpUrl does that too, but it's 
-			// more userfriendly on errors (more exact error message)
-			if (url.SetHttpUrl(sLine))
-			{
-				_FixPostPreSlashes(url.sPath);
-				pair<tHttpUrl, const string*> info(url,
-						GetRepoEntryNamePtr(sRepName));
-				mapUrl2pVname.insert(info);
-			}
-			else
-			{
-				cout << "Parse error, invalid URL " << sLine
-						<< " in line " << sFile << ":"<< reader.GetCurrentLine() << endl;
+		if (0==sLine.compare(0, 1, "#"))
+			continue;
 
-				exit(2);
-			}
+		if (url.SetHttpUrl(sLine))
+		{
+			_FixPostPreSlashes(url.sPath);
+			pair<tHttpUrl, const string*> info(url, GetRepoEntryNamePtr(sRepName));
+			mapUrl2pVname.insert(info);
+#ifdef DEBUG
+						cerr << "Mapping: "<< url.ToURI(false)
+						<< " -> "<< sRepName <<endl;
+#endif
 			continue;
 		}
-		else if (sLine.empty()) // end of block, eof, ... -> commit it
+
+		if (sLine.empty()) // end of block, eof, ... -> commit it
 		{
 			if (hosts.empty() && paths.empty())
-				continue; // dummy run
+				continue; // dummy run or whitespace in a URL style list
 			if ( !hosts.empty() && paths.empty())
 			{
 				cerr << "Warning, missing path spec for the site " << hosts[0] <<", ignoring mirror."<< endl;
@@ -974,9 +888,8 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 			}
 			if ( !paths.empty() && hosts.empty())
 			{
-				cout << "Parse error, missing Site: field around line "
-						<< sFile << ":"<< reader.GetCurrentLine() << endl;
-				exit(2);
+				BARF("Parse error, missing Site: field around line "
+						<< sFile << ":"<< reader.GetCurrentLine());
 			}
 			for (tStrVecIterConst itHost=hosts.begin();
 			itHost!=hosts.end(); 
@@ -994,7 +907,7 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 					mapUrl2pVname.insert(info);
 
 #ifdef DEBUG
-						cout << "Mapping: "<< *itHost << *itPath 
+						cerr << "Mapping: "<< *itHost << *itPath
 						<< " -> "<< sRepName <<endl;
 #endif
 				}
@@ -1003,15 +916,14 @@ void ReadRewriteFile(const string & sFile, const string & sRepName)
 			paths.clear();
 			continue;
 		}
-		else if(!ParseKeyValLine(sLine, key, val))
-		{
-			cerr << "Error parsing rewrite definitions, around line "
-					<< sFile << ":"<< reader.GetCurrentLine() << endl;
 
-			exit(1);
+		if(!ParseKeyValLine(sLine, key, val))
+		{
+			BARF("Error parsing rewrite definitions, around line "
+					<< sFile << ":"<< reader.GetCurrentLine() << " : " << sLine);
 		}
 		
-		// got something, intepret it...
+		// got something, interpret it...
 		if( keyEq("Site", key) || keyEq("Alias", key) || keyEq("Aliases", key))
 			Tokenize(val, SPACECHARS, hosts, true);
 		
@@ -1040,30 +952,30 @@ tRepoData::~tRepoData()
 	}
 }
 
-void ReadConfigDirectory(const char *szPath)
+void ReadConfigDirectory(const char *szPath, bool bTestMode)
 {
-	// TODO: early abort when the dir does not exist!
-
 	char buf[PATH_MAX];
+	g_testMode=bTestMode;
 	if(!realpath(szPath, buf))
-  {
-     cerr << "Failed to open config directory" << endl;
-     exit(42);
-  }
+		BARF("Failed to open config directory");
+
 	confdir=buf; // pickup the last config directory
 
 #if defined(HAVE_WORDEXP) || defined(HAVE_GLOB)
 	tStrDeq srcs=ExpandFilePattern(confdir+SZPATHSEP"*.conf", true);
 	for(tStrDeq::const_iterator it=srcs.begin(); it!=srcs.end(); it++)
-		_ReadMainConfiguration(*it);
+		ReadOneConfFile(*it);
 #else
-	_ReadMainConfiguration(confdir+SZPATHSEP"acng.conf");
+	ReadOneConfFile(confdir+SZPATHSEP"acng.conf");
 #endif
 }
 
 void PostProcConfig() 
 {
 	
+	if(port.empty()) // heh?
+		port=ACNG_DEF_PORT;
+
 	// let's also apply the umask to the directory permissions
 	{
 		mode_t mask = umask(0);
@@ -1079,18 +991,12 @@ void PostProcConfig()
 	{
 		mode_t nUmask=0;
 		if(sUmask.size()>4)
-		{
-			cerr << "Invalid umask length\n";
-			exit(EXIT_FAILURE);
-		}
+			BARF("Invalid umask length\n");
 		for(unsigned int i=0; i<sUmask.size(); i++)
 		{
 			unsigned int val = sUmask[sUmask.size()-i-1]-'0';
 			if(val>7)
-			{
-				cerr << "Invalid umask value\n" <<endl;
-				exit(EXIT_FAILURE);
-			}
+				BARF("Invalid umask value\n");
 			nUmask |= (val<<(3*i));
 		
 		}
@@ -1099,16 +1005,11 @@ void PostProcConfig()
 	}
 #endif
 	
-   if(cachedir.empty() || cachedir[0] != CPATHSEP) {
-      cerr << "Cache directory unknown or not absolute, terminating..." <<endl;
-      exit(EXIT_FAILURE);
-   }
+   if(cachedir.empty() || cachedir[0] != CPATHSEP)
+	   BARF("Cache directory unknown or not absolute, terminating...");
    
    if(!rechecks::CompileExpressions())
-   {
-	   cerr << "An error occured while compiling file type regular expression!" <<endl;
-	   exit(EXIT_FAILURE);
-   }
+	   BARF("An error occured while compiling file type regular expression!");
    
    if(acfg::tpthreadmax < 0)
 	   acfg::tpthreadmax = MAX_VAL(int);
@@ -1120,10 +1021,7 @@ void PostProcConfig()
 	cacheDirSlash=cachedir+CPATHSEP;
 
    if(!pidfile.empty() && pidfile.at(0) != CPATHSEP)
-   {
-	   cerr << "Pid file path must be absolute, terminating..."  <<endl;
-	         exit(EXIT_FAILURE);
-   }
+	   BARF("Pid file path must be absolute, terminating...");
    
    if(!acfg::agentname.empty())
 	   acfg::agentheader=string("User-Agent: ")+acfg::agentname + "\r\n";
@@ -1164,8 +1062,7 @@ void PostProcConfig()
 		   tmpDontcacherq.empty() ? tmpDontcache : tmpDontcacherq,
 		   tmpDontcacheResolved.empty() ? tmpDontcache : tmpDontcacheResolved))
    {
-	   cerr << "An error occurred while compiling regular expression for non-cached paths!" <<endl;
-	   exit(EXIT_FAILURE);
+	   BARF("An error occurred while compiling regular expression for non-cached paths!");
    }
    tmpDontcache.clear();
    tmpDontcacheResolved.clear();
@@ -1190,13 +1087,18 @@ void PostProcConfig()
 	   redirmax = forcemanaged ? 0 : ACFG_REDIRMAX_DEFAULT;
 
    if(!persistoutgoing)
-	   pipelinelen = 0;
+	   pipelinelen = 1;
+
+   if(!pipelinelen) {
+	   cerr << "Warning, remote pipeline depth of 0 makes no sense, assuming 1." << endl;
+	   pipelinelen = 1;
+   }
 
    if (acfg::debug >= LOG_MORE)
 	{
 		for (UINT i = 0; i < _countof(n2sTbl); i++)
 			if (n2sTbl[i].ptr)
-				cout << n2sTbl[i].name << " = " << *(n2sTbl[i].ptr) << endl;
+				cerr << n2sTbl[i].name << " = " << *(n2sTbl[i].ptr) << endl;
 
 		if (acfg::debug >= LOG_DEBUG)
 		{
@@ -1204,24 +1106,24 @@ void PostProcConfig()
 			for (UINT i = 0; i < _countof(n2sTbl); i++)
 				if (n2sTbl[i].ptr)
 				{
-					cout << n2sTbl[i].name << " = ";
+					cerr << n2sTbl[i].name << " = ";
 					for (const char *p = n2sTbl[i].ptr->c_str(); *p; p++)
 						if ('\\' == *p)
-							cout << "\\\\";
+							cerr << "\\\\";
 						else
-							cout << *p;
-					cout << endl;
+							cerr << *p;
+					cerr << endl;
 				}
 		}
 
 		for (UINT i = 0; i < _countof(n2iTbl); i++)
 			if (n2iTbl[i].ptr)
-				cout << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
+				cerr << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
 	}
 
 #ifndef DEBUG
    if(acfg::debug >= LOG_DEBUG)
-	   cout << "\n\nAdditional debugging information not compiled in.\n\n";
+	   cerr << "\n\nAdditional debugging information not compiled in.\n\n";
 #endif
    
 #if 0 //def DEBUG
@@ -1249,13 +1151,14 @@ void PostProcConfig()
 			   cout << jit->ToURI() <<endl;
 		   }
 	   }
-	   */
+
 	   for(tUrl2RepIter it=mapUrl2pVname.begin(); it!=mapUrl2pVname.end(); it++)
 	   {
 		   cout << it->first.ToURI(false) << " ___" << *(it->second) << endl;
 	   }
 
 	   exit(1);
+	   */
    }
 } // PostProcConfig
 
