@@ -12,7 +12,8 @@ namespace acfg
 {
 
 string cachedir("/var/tmp"), logdir("/var/tmp"), fifopath, pidfile, reportpage,
-confdir, adminauth, bindaddr, mirrorsrcs, suppdir("/usr/lib/apt-cacher-ng");
+confdir, adminauth, bindaddr, mirrorsrcs, suppdir("/usr/lib/apt-cacher-ng"),
+capath("/etc/ssl/certs"), cafile;
 
 #define INFOLDER "(^|.*/)"
 #define COMPRLIST "(\\.gz|\\.bz2|\\.lzma|\\.xz)"
@@ -65,7 +66,8 @@ keepnver(0), maxtempdelay(27), vrangeops(1);
 int dlbufsize(70000), exfailabort(1), exporigin(false), numcores(1),
 logxff(false), oldupdate(false), recompbz2(false), nettimeout(60), updinterval(0),
 forwardsoap(RESERVED_DEFVAL), usewrap(RESERVED_DEFVAL), redirmax(RESERVED_DEFVAL),
-stucksecs(500), persistoutgoing(1), pipelinelen(255), exsupcount(RESERVED_DEFVAL);
+stucksecs(500), persistoutgoing(1), pipelinelen(255), exsupcount(RESERVED_DEFVAL),
+optproxytimeout(-1);
 
 #ifdef DEBUG
 int dnscachetime(30);
@@ -74,7 +76,7 @@ int dnscachetime(1800);
 #endif
 
 string agentname("Debian Apt-Cacher-NG/" ACVERSION);
-string remoteport("80"), port("3142");
+string remoteport("80"), port(ACNG_DEF_PORT);
 string agentheader;
 
 string requestapx;
