@@ -19,16 +19,31 @@ using namespace MYSTD;
 #include <signal.h>
 #include <errno.h>
 
-//#include "job.h"
+#ifdef HAVE_SSL
+/* OpenSSL headers */
+
+#include "openssl/bio.h"
+#include "openssl/ssl.h"
+#include "openssl/err.h"
+
+
+#endif
+
+
+#ifdef DEBUG
+#include "filereader.h"
+#include "csmapping.h"
+#endif
 
 static void usage();
-static void _TestCacheDir();
+static void SetupCacheDir();
 void term_handler(int signum);
 void log_handler(int signum);
 void dump_handler(int signum);
 void check_algos();
 
 void DispatchAndRunMaintTask(cmstring &cmd, int fd, const char *auth);
+int wcat(LPCSTR url, LPCSTR proxy);
 
 #ifdef HAVE_DAEMON
 inline bool fork_away()
@@ -62,6 +77,15 @@ inline bool fork_away()
 int main(int argc, char **argv)
 {
 
+#ifdef HAVE_SSL
+	SSL_load_error_strings();
+	ERR_load_BIO_strings();
+	ERR_load_crypto_strings();
+ 	ERR_load_SSL_strings();
+	OpenSSL_add_all_algorithms();
+	SSL_library_init();
+#endif
+
 
 // PLAYGROUND
 
@@ -88,19 +112,25 @@ int main(int argc, char **argv)
 	printf("\n");
 	exit(0);
 */
-	/*
-	uint8_t csum[16];
-	string s(argv[1]);
-	filereader::GetMd5Sum(s, csum);
-	for(int i=0;i<16;i++)
+
+#ifdef DEBUG
+	if (getenv("GETSUM"))
+	{
+		uint8_t csum[20];
+		string s(getenv("GETSUM"));
+		off_t resSize;
+		bool ok=filereader::GetChecksum(s, CSTYPE_SHA1, csum, true, resSize, stdout);
+		for (UINT i = 0; i < sizeof(csum); i++)
 			printf("%02x", csum[i]);
 		printf("\n");
-		if(CsEqual((unsigned char*) argv[2], csum))
-			printf("Is OK\n");
-		else
-			printf("Differenz\n");
+		if (ok && getenv("REFSUM"))
+		{
+			printf(CsEqual(getenv("REFSUM"), csum, sizeof(csum))
+			? "IsOK\n" : "Diff\n");
+		}
 		exit(0);
-	*/
+	}
+
 	/*
 	bool Bz2compressFile(const char *, const char*);
 	return ! Bz2compressFile(argv[1], argv[2]);
@@ -111,12 +141,26 @@ int main(int argc, char **argv)
 	MYSTD::cerr << tbuf << MYSTD::endl;
 	exit(1);
 */
-	const char *toBase64=getenv("TOBASE64");
-	if(toBase64)
+#endif
+
+	const char *envvar=getenv("TOBASE64");
+	if(envvar)
 	{
-		MYSTD::cout << acfg::EncodeBase64Auth(toBase64);
+		MYSTD::cout << EncodeBase64Auth(envvar);
 		return 0;
 	}
+	envvar=getenv("BECURL");
+	if(envvar)
+		return wcat(envvar, getenv("http_proxy"));
+
+#ifdef DEBUG
+	envvar=getenv("PARSEIDX");
+	if(envvar)
+	{
+		int parseidx_demo(LPCSTR);
+		return parseidx_demo(envvar);
+	}
+#endif
 
 	check_algos();
 	struct sigaction act;
@@ -143,56 +187,59 @@ int main(int argc, char **argv)
 	sigaction(SIGXFSZ, &act, NULL);
 #endif
 	
+	// preprocess some startup related parameters
 	bool bForceCleanup(false);
-    for (int i=1; i<argc; i++)
+    for (char **p=argv+1; p<argv+argc; p++)
     {
-
-		if (!strncmp(argv[i], "-h", 2))
+		if (!strncmp(*p, "-h", 2))
 			usage();
-
-		if (!strncmp(argv[i], "-v", 2))
+		else if (!strncmp(*p, "-v", 2))
 		{
 			acfg::debug=acfg::debug|LOG_DEBUG|LOG_MORE;
-			*(argv[i])=0x0; // ignore if ever checked
+			**p=0x0; // ignore it if ever checked anywhere
 		}
-
-		if (!strncmp(argv[i], "-e", 2))
+		else if (!strncmp(*p, "-e", 2))
 		{
 			bForceCleanup=true;
-			*(argv[i])=0x0; // ignore if ever checked
+			**p=0x0; // ignore it if ever checked anywhere
 		}
     }
     
-    for (int i=1; i<argc; i++)
-    {
-    	
-        if(!strcmp(argv[i], "-c"))
+    LPCSTR PRINTCFGVAR=getenv("PRINTCFGVAR");
+
+    for (char **p=argv+1; p<argv+argc; p++)
+     {
+        if(!strcmp(*p, "-c"))
         {
-        	i++;
-        	if(i<argc)
-        		acfg::ReadConfigDirectory(argv[i]);
+        	++p;
+        	if(p < argv+argc)
+        		acfg::ReadConfigDirectory(*p, PRINTCFGVAR);
         	else
         		usage();
 		}
-		else if(! *(argv[i]))
-			continue;
-		else
+		else if(**p) // not empty
 		{
-			if(!acfg::SetOption(argv[i], false))
+			if(!acfg::SetOption(*p, false))
 				usage();
 		}
-
 	}
     
+    if(PRINTCFGVAR)
+    {
+    	acfg::printVar(PRINTCFGVAR);
+    	return 0;
+    }
+
     if(!aclog::open())
     {
-        cerr << "Problem creating log files. Check permissions of the log directory, " << acfg::logdir<<endl;
+        cerr << "Problem creating log files. Check permissions of the log directory, "
+        		<< acfg::logdir<<endl;
         exit(1);
     }
 
 	acfg::PostProcConfig();
     
-    _TestCacheDir();
+    SetupCacheDir();
 
 	extern mstring sReplDir;
 	DelTree(acfg::cacheDirSlash+sReplDir);
@@ -211,7 +258,8 @@ int main(int argc, char **argv)
 
 	if (!fork_away())
 	{
-		cerr << "Failed to change to daemon mode" << endl;
+		errnoFmter ef("Failed to change to daemon mode");
+		cerr << ef << endl;
 		exit(43);
 	}
 
@@ -247,12 +295,21 @@ static void usage() {
 }
 
 
-static void _TestCacheDir()
+static void SetupCacheDir()
 {
+	using namespace acfg;
+	if(!Cstat(cacheDirSlash))
+	{
+		// well, attempt to create it then
+		mstring path=cacheDirSlash+'/';
+		for(UINT pos=0; (pos=path.find(SZPATHSEP, pos)) < path.size(); ++pos)
+            mkdir((const char*) path.substr(0,pos).c_str(), (UINT) dirperms);
+	}
+
 	struct timeval tv;
 	gettimeofday(&tv, NULL);
 	tSS buf;
-	buf << acfg::cacheDirSlash << "testfile." <<42*tv.tv_usec*tv.tv_sec;
+	buf << cacheDirSlash << "testfile." <<42*tv.tv_usec*tv.tv_sec;
 	mkbasedir(buf.c_str()); // try or force its directory creation
 	int t=open( buf.c_str(), O_WRONLY | O_CREAT | O_TRUNC, 00644);
 	if (t>=0)
@@ -261,8 +318,8 @@ static void _TestCacheDir()
 		if(0==unlink(buf.c_str()))
 			return;
 	}
-	cerr << "Cache directory not writable. Check the permissions of "
-		<< acfg::cachedir << "!" <<endl;
+	cerr << "Failed to create cache directory or directory not writable." << endl
+			<< "Check the permissions of " << cachedir << "!" << endl;
 
 	exit(1);
 }
@@ -301,3 +358,58 @@ void term_handler(int signum)
 		return;
 	}
 }
+
+#include "dlcon.h"
+#include "fileio.h"
+#include "fileitem.h"
+
+int wcat(LPCSTR surl, LPCSTR proxy)
+{
+
+	acfg::dnscachetime=0;
+	acfg::persistoutgoing=0;
+
+	if(proxy)
+		if(acfg::proxy_info.SetHttpUrl(proxy))
+			return -1;
+	tHttpUrl url;
+	if(!surl || !url.SetHttpUrl(surl))
+		return -2;
+	dlcon dl(true);
+
+	class tPrintItem : public fileitem
+	{
+	public:
+		tPrintItem()
+		{
+			m_bAllowStoreData=false;
+			m_nSizeChecked = m_nSizeSeen = 0;
+		};
+		virtual FiStatus Setup(bool)
+		{
+			m_nSizeChecked = m_nSizeSeen = 0;
+			return m_status = FIST_INITED;
+		}
+		virtual int GetFileFd() { return 1; }; // something, don't care for now
+		virtual bool DownloadStartedStoreHeader(const header & h, const char *,
+				bool, bool&)
+		{
+			return true;
+		}
+		virtual bool StoreFileData(const char *data, unsigned int size)
+		{
+			return (size==fwrite(data, sizeof(char), size, stdout));
+		}
+		ssize_t SendData(int , int, off_t &, size_t )
+		{
+			return 0;
+		}
+	};
+
+	tFileItemPtr fi((fileitem*)new tPrintItem);
+	dl.AddJob(fi, url);
+	dl.WorkLoop();
+	return (fi->WaitForFinish(NULL) == fileitem::FIST_COMPLETE
+			&& fi->GetHeaderUnlocked().getStatus() == 200) ? 0 : -3;
+}
+
