@@ -17,7 +17,8 @@ using namespace MYSTD;
 static const cmstring sGenericError("567 Unknown download error occured");
 
 dlcon::dlcon(bool bManualExecution, string *xff) :
-		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nTempPipelineDisable(0)
+		m_bStopASAP(false), m_bManualMode(bManualExecution), m_nTempPipelineDisable(0),
+		m_proxied(!acfg::proxy_info.sHost.empty())
 {
 	LOGSTART("dlcon::dlcon");
 	m_wakepipe[0] = m_wakepipe[1] = -1;
@@ -50,15 +51,9 @@ struct tDlJob
 #define HINT_KILL_LAST_FILE 128
 #define HINT_TGTCHANGE 256
 
-	inline const mstring GetPeerName()
+	inline const tHttpUrl & GetPeerHost()
 	{
-		return m_pCurBackend ? m_pCurBackend->sHost : m_fileUri.sHost;
-	}
-
-	inline const MYSTD::string GetPeerPort()
-	{
-		cmstring &ps = m_pCurBackend ? m_pCurBackend->sPort : m_fileUri.sPort;
-		return ps.empty() ? acfg::remoteport : ps;
+		return m_pCurBackend ? *m_pCurBackend : m_fileUri;
 	}
 
 	inline acfg::tRepoData::IHookHandler * GetConnStateTracker()
@@ -150,7 +145,9 @@ struct tDlJob
 
 		m_pBEdata = NULL;
 		m_pCurBackend = NULL;
-		if (!m_fileUri.SetHttpUrl(s))
+		if (m_fileUri.SetHttpUrl(s))
+			m_fileUri.bIsTransferlEncoded=true;
+		else
 		{
 			sErrorMsg = "500 Bad redirection (invalid URL)";
 			return false;
@@ -173,34 +170,48 @@ struct tDlJob
 //}
 
 // needs connectedHost, blacklist, output buffer from the parent, proxy mode?
-	inline void AppendRequest(tSS &head, cmstring &xff)
+	inline void AppendRequest(tSS &head, cmstring &xff, bool proxied)
 	{
 		LOGSTART("tDlJob::AppendRequest");
-		bool proxied = ! acfg::proxy_info.sHost.empty();
 
 		head << (m_pStorage->m_bHeadOnly ? "HEAD " : "GET ");
-		if (!proxied) // use absolute path
+
+		if (proxied)
+			head << RemoteUri(true);
+		else // only absolute path without scheme
 		{
 			if (m_pCurBackend) // base dir from backend definition
 				head << UrlEscape(m_pCurBackend->sPath);
-			head << UrlEscape(m_fileUri.sPath);
+
+			if(m_fileUri.bIsTransferlEncoded)
+				head << m_fileUri.sPath;
+			else
+				head << UrlEscape(m_fileUri.sPath);
 		}
-		else
-			head << RemoteUri(true);
 
 		ldbg(RemoteUri(true));
 
-		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerName() << "\r\n";
+		head << " HTTP/1.1\r\n" << acfg::agentheader << "Host: " << GetPeerHost().sHost << "\r\n";
 
 		if (proxied) // proxy stuff, and add authorization if there is any
 		{
 			ldbg("using proxy");
-			head << acfg::proxy_info.sPath
-					// Proxy-Connection is a non-sensical copy of Connection but some proxy
-					// might listen only to this one so better add it
-					<< (acfg::persistoutgoing ? "Proxy-Connection: keep-alive\r\n"
+			if(!acfg::proxy_info.sUserPass.empty())
+			{
+				head << "Proxy-Authorization: Basic "
+						<< EncodeBase64Auth(acfg::proxy_info.sUserPass) << "\r\n";
+			}
+			// Proxy-Connection is a non-sensical copy of Connection but some proxy
+			// might listen only to this one so better add it
+			head << (acfg::persistoutgoing ? "Proxy-Connection: keep-alive\r\n"
 									: "Proxy-Connection: close\r\n");
+		}
 
+		const tHttpUrl *pCredSrc=m_pCurBackend ? m_pCurBackend : &m_fileUri;
+		if(pCredSrc && ! pCredSrc->sUserPass.empty())
+		{
+			head << "Authorization: Basic "
+				<< EncodeBase64Auth(pCredSrc->sUserPass) << "\r\n";
 		}
 
 		// either by backend or by host in file uri, never both
@@ -283,9 +294,15 @@ struct tDlJob
 
 	inline string RemoteUri(bool bEscaped)
 	{
-		return m_pCurBackend
-				? m_pCurBackend->ToURI(bEscaped) + m_fileUri.sPath
-						: m_fileUri.ToURI(bEscaped);
+		if(m_pCurBackend)
+			return m_pCurBackend->ToURI(bEscaped) +
+					( (bEscaped && !m_fileUri.bIsTransferlEncoded)
+							?
+							UrlEscape(m_fileUri.sPath)
+							:
+							m_fileUri.sPath);
+
+		return m_fileUri.ToURI(bEscaped);
 	}
 
 	uint_fast8_t NewDataHandler(acbuf & inBuf)
@@ -325,7 +342,7 @@ struct tDlJob
 	 *
 	 * Process new incoming data and write it down to disk or other receivers.
 	 */
-	UINT ProcessIncomming(acbuf & inBuf, bool bOnlyRedirections)
+	UINT ProcessIncomming(acbuf & inBuf, bool bOnlyRedirectionActivity)
 	{
 		LOGSTART("tDlJob::ProcessIncomming");
 		if (!m_pStorage)
@@ -372,20 +389,30 @@ struct tDlJob
 
 				int st = h.getStatus();
 
-				// special handling for redirection, see caller ExchangeData() further processing
-				if(st == 301 || st == 302 || st == 307)
+				if (acfg::redirmax) // internal redirection might be disabled
 				{
-					if(!acfg::redirmax)
+					if (st == 301 || st == 302 || st == 307)
 					{
-						ldbg("Internal redirection disabled by administrator, forward it to the user");
+						if (RewriteSource(h.h[header::LOCATION]))
+						{
+							// drop the redirect page contents if possible so the outer loop
+							// can scan other headers
+							off_t contLen = atoofft(h.h[header::CONTENT_LENGTH], 0);
+							if (contLen <= inBuf.size())
+								inBuf.drop(contLen);
+							return HINT_TGTCHANGE; // no other flags, caller will evaluate the state
+						}
+						else
+							return EFLAG_JOB_BROKEN;
 					}
-					else if(RewriteSource(h.h[header::LOCATION]))
-						return HINT_TGTCHANGE;
-					else
-						return EFLAG_JOB_BROKEN;
+
+					// for non-redirection responses process as usual
+
+					// unless it's a probe run from the outer loop, in this case we
+					// should go no further
+					if (bOnlyRedirectionActivity)
+						return EFLAG_LOST_CON | HINT_DISCON;
 				}
-				else if(bOnlyRedirections)
-					return EFLAG_LOST_CON; // any error, just not tgtchange or broken
 
 				// explicitly blacklist mirror if key file is missing
 				if (st >= 400 && m_pBEdata)
@@ -544,10 +571,9 @@ private:
 
 inline void dlcon::BlacklistMirror(tDlJobPtr & job, cmstring &msg)
 {
-	LOGSTART2("dlcon::BlacklistMirror", "what? [" <<
-			job->GetPeerName() << "]:" <<
-			job->GetPeerPort());
-	m_blacklist[make_pair(job->GetPeerName(), job->GetPeerPort())] = msg;
+	LOGSTART2("dlcon::BlacklistMirror", "blacklisting " <<
+			job->GetPeerHost().ToURI(false));
+	m_blacklist[make_pair(job->GetPeerHost().sHost, job->GetPeerHost().GetPort())] = msg;
 }
 
 
@@ -564,8 +590,8 @@ inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 		// keep the existing one if possible
 		if (job->m_pCurBackend)
 		{
-			LOG("Checking [" << job->m_pCurBackend->sHost << "]:" << job->m_pCurBackend->sPort);
-			bliter = m_blacklist.find(make_pair(job->m_pCurBackend->sHost, job->m_pCurBackend->sPort));
+			LOG("Checking [" << job->m_pCurBackend->sHost << "]:" << job->m_pCurBackend->GetPort());
+			bliter = m_blacklist.find(make_pair(job->m_pCurBackend->sHost, job->m_pCurBackend->GetPort()));
 			if(bliter == m_blacklist.end())
 				return true;
 		}
@@ -573,7 +599,7 @@ inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 		for (vector<tHttpUrl>::const_iterator it=job->m_pBEdata->m_backends.begin();
 				it!=job->m_pBEdata->m_backends.end(); ++it)
 		{
-			bliter = m_blacklist.find(make_pair(it->sHost, it->sPort));
+			bliter = m_blacklist.find(make_pair(it->sHost, it->GetPort()));
 			if(bliter == m_blacklist.end())
 			{
 				job->m_pCurBackend = &(*it);
@@ -587,7 +613,7 @@ inline bool dlcon::SetupJobConfig(tDlJobPtr &job, mstring *pReasonMsg)
 	}
 
 	// ok, look for the mirror data itself
-	bliter = m_blacklist.find(make_pair(job->GetPeerName(), job->GetPeerPort()));
+	bliter = m_blacklist.find(make_pair(job->GetPeerHost().sHost, job->GetPeerHost().GetPort()));
 	if(bliter == m_blacklist.end())
 		return true;
 	else
@@ -681,6 +707,12 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 				ldbg("Needs to send " << m_sendBuf.size() << " bytes");
 				FD_SET(fd, &wfds);
 			}
+#ifdef HAVE_SSL
+			else if(con->GetBIO() && BIO_should_write(con->GetBIO()))
+			{
+				FD_SET(fd, &wfds);
+			}
+#endif
 		}
 
 		ldbg("select dlcon");
@@ -704,7 +736,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 #ifdef MINIBUILD
 			string fer("select failed");
 #else
-			aclog::errnoFmter fer("FAILURE: select, ");
+			errnoFmter fer("FAILURE: select, ");
 			LOG(fer);
 #endif
 			sErrorMsg = string("500 Internal malfunction, ") + fer;
@@ -732,31 +764,58 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 			return HINT_SWITCH;
 		}
 
-		if (fd>=0 && FD_ISSET(fd, &wfds))
+		if (fd>=0)
 		{
-			FD_CLR(fd, &wfds);
-
-			ldbg("Sending data...\n" << m_sendBuf);
-			int s = ::send(fd, m_sendBuf.data(), m_sendBuf.length(), MSG_NOSIGNAL);
-			ldbg("Sent " << s << " bytes from " << m_sendBuf.length() << " to " << con.get() );
-			if (s < 0)
+#ifdef HAVE_SSL
+			if (con->GetBIO())
 			{
-				// EAGAIN is weird but let's retry later, otherwise reconnect
-				if (errno != EAGAIN && errno != EINTR)
-				{
-					sErrorMsg = "502 Send failed";
-					return EFLAG_LOST_CON;
-				}
+				int s=BIO_write(con->GetBIO(), m_sendBuf.rptr(), m_sendBuf.size());
+				if(s>0)
+					m_sendBuf.drop(s);
 			}
 			else
-				m_sendBuf.drop(s);
+#endif
+			if (FD_ISSET(fd, &wfds))
+			{
+				FD_CLR(fd, &wfds);
+
+				ldbg("Sending data...\n" << m_sendBuf);
+				int s = ::send(fd, m_sendBuf.data(), m_sendBuf.length(), MSG_NOSIGNAL);
+				ldbg("Sent " << s << " bytes from " << m_sendBuf.length() << " to " << con.get());
+				if (s < 0)
+				{
+					// EAGAIN is weird but let's retry later, otherwise reconnect
+					if (errno != EAGAIN && errno != EINTR)
+					{
+						sErrorMsg = "502 Send failed";
+						return EFLAG_LOST_CON;
+					}
+				}
+				else
+					m_sendBuf.drop(s);
+			}
 		}
 
-		if (fd >=0 && FD_ISSET(fd, &rfds))
+		if (fd >=0 && (FD_ISSET(fd, &rfds)
+#ifdef HAVE_SSL
+				|| (con->GetBIO() && BIO_should_read(con->GetBIO()))
+#endif
+			))
 		{
-
-			r = m_inBuf.sysread(fd);
-
+#ifdef HAVE_SSL
+			if(con->GetBIO())
+			{
+				r=BIO_read(con->GetBIO(), m_inBuf.wptr(), m_inBuf.freecapa());
+				if(r>0)
+					m_inBuf.got(r);
+				else // <=0 doesn't mean an error, only a double check can tell
+					r=BIO_should_read(con->GetBIO()) ? 1 : -errno;
+			}
+			else
+#endif
+			{
+				r = m_inBuf.sysread(fd);
+			}
 
 #ifdef DISCO_FAILURE
 #warning hej
@@ -791,7 +850,7 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 				sErrorMsg = "502 EPIC FAIL";
 #else
 				// pickup the error code for later and kill current connection ASAP
-				sErrorMsg = aclog::errnoFmter("502 ");
+				sErrorMsg = errnoFmter("502 ");
 #endif
 				return EFLAG_LOST_CON;
 			}
@@ -848,20 +907,24 @@ inline UINT dlcon::ExchangeData(mstring &sErrorMsg, tTcpHandlePtr &con, tDljQueu
 
 				if (HINT_TGTCHANGE & res)
 				{
-					/* target was rewritten by the job handler, but there might be more redirections
-				    in the pipe which we can analyze and use ASAP to avoid later requests.
-				    Process as much as we can (carefully). */
+					/* If the target was modified for internal redirection then there might be
+					 * more responses of that kind in the queue. Apply the redirection handling
+					 * to the rest as well if possible without having side effects.
+					 */
 					tDljQueue::iterator it = inpipe.begin();
 					for(++it; it != inpipe.end(); ++it)
 					{
 						UINT rr = (**it).ProcessIncomming(m_inBuf, true);
+						// just the internal rewriting applied and nothing else?
 						if( HINT_TGTCHANGE != rr )
 						{
+							// not internal redirection or some failure doing it
 							m_nTempPipelineDisable=30;
 							return (HINT_TGTCHANGE|HINT_DISCON);
 						}
 					}
-					return HINT_TGTCHANGE;
+					// processed all inpipe stuff but if the buffer is still not empty then better disconnect
+					return HINT_TGTCHANGE | (m_inBuf.empty() ? 0 : HINT_DISCON);
 				}
 
 				// else case: error handling, pass to main loop
@@ -899,8 +962,9 @@ void dlcon::WorkLoop()
 {
 	LOGSTART("dlcon::WorkLoop");
     string sErrorMsg;
+    m_inBuf.clear();
     
-	if (!m_inBuf.init(acfg::dlbufsize))
+	if (!m_inBuf.setsize(acfg::dlbufsize))
 	{
 		aclog::err("500 Out of memory");
 		return;
@@ -923,9 +987,6 @@ void dlcon::WorkLoop()
 
 	while(true) // outer loop: jobs, connection handling
 	{
-		// helper var, remember that the connection was recycled before
-		bool bUsed=false;
-
         // init state or transfer loop jumped out, what are the needed actions?
         {
         	setLockGuard;
@@ -954,6 +1015,7 @@ void dlcon::WorkLoop()
         		// cleanup after the last connection - send buffer, broken jobs, ...
         		m_sendBuf.clear();
         		m_inBuf.clear();
+        		inpipe.clear();
 
         		bStopRequesting=false;
 
@@ -973,14 +1035,29 @@ void dlcon::WorkLoop()
         			LOG("no jobs left, start waiting")
         			goto go_select; // nothing left, might receive new jobs soon
         		}
-        		if(acfg::proxy_info.sHost.empty())
-        			con = tcpconnect::CreateConnected(m_qNewjobs.front()->GetPeerName(),
-        				m_qNewjobs.front()->GetPeerPort(), sErrorMsg,
-        				&bUsed, m_qNewjobs.front()->GetConnStateTracker());
+
+        		bool bUsed=false;
+#ifdef HAVE_SSL
+#define doconnect(x, t) tcpconnect::CreateConnected(x.sHost, x.GetPort(), \
+				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), x.bSSL, t)
+#else
+#define doconnect(x, t) tcpconnect::CreateConnected(x.sHost, x.GetPort(), \
+				sErrorMsg, &bUsed, m_qNewjobs.front()->GetConnStateTracker(), false, t)
+
+#endif
+
+        		if(!m_proxied)
+        			con = doconnect(m_qNewjobs.front()->GetPeerHost(), acfg::nettimeout);
         		else
-        			con = tcpconnect::CreateConnected(acfg::proxy_info.sHost,
-        					acfg::proxy_info.sPort, sErrorMsg, &bUsed,
-        					m_qNewjobs.front()->GetConnStateTracker());
+        			con = doconnect(acfg::proxy_info,
+        				acfg::optproxytimeout>0 ? acfg::optproxytimeout : acfg::nettimeout);
+
+        		if(!con && acfg::optproxytimeout>0)
+        		{
+        			ldbg("optional proxy timed out, disable");
+        			m_proxied=false;
+        			con = doconnect(m_qNewjobs.front()->GetPeerHost(), acfg::nettimeout);
+        		}
 
         		nLostConTolerance = MAX_RETRY + bUsed;
 
@@ -989,6 +1066,14 @@ void dlcon::WorkLoop()
         		if(con)
         		{
         			ldbg("target? [" << con->GetHostname() << "]:" << con->GetPort());
+
+        			// must test this connection, just be sure no crap is in the pipe
+        			if (bUsed && check_read_state(con->GetFD()))
+        			{
+        				ldbg("code: MoonWalker");
+        				con.reset();
+        				continue;
+        			}
         		}
         		else
         		{
@@ -998,15 +1083,19 @@ void dlcon::WorkLoop()
         	}
 
         	// connection should be stable now, prepare all jobs and/or move to pipeline
-        	while(!bStopRequesting && !m_qNewjobs.empty())
+        	while(!bStopRequesting
+        			&& !m_qNewjobs.empty()
+        			&& int(inpipe.size()) <= acfg::pipelinelen)
         	{
-        		tDlJobPtr &cjob = m_qNewjobs.front();
+   				tDlJobPtr &cjob = m_qNewjobs.front();
 
         		bool bGoodConfig = SetupJobConfig(cjob, NULL);
 
+        		/*
         		ldbg("target: " << cjob->GetPeerName() << " vs " << con->GetHostname()
         				<< ", ports: " << cjob->GetPeerPort() << " vs " << con->GetPort()
         				<< ", good config: " << bGoodConfig);
+*/
 
         		if(!bGoodConfig)
         		{
@@ -1015,34 +1104,32 @@ void dlcon::WorkLoop()
         			continue;
         		}
 
-        		if(acfg::proxy_info.sHost.empty()
+        		// direct/non-proxy mode and the wanted target host changed?
+        		if(!m_proxied
         				// needs to send them for the connected target host
-        				&& ( cjob->GetPeerName() != con->GetHostname()
-        				|| cjob->GetPeerPort() != con->GetPort()) )
+        				&& ( cjob->GetPeerHost().sHost != con->GetHostname()
+        				|| cjob->GetPeerHost().GetPort() != con->GetPort()) )
         		{
-        			LOG("host mismatch, stop sending requesting for now");
+        			LOG("host mismatch," << cjob->GetPeerHost().sHost << ":" <<
+        					cjob->GetPeerHost().GetPort() <<
+        					" vs. " << con->GetHostname() << ":"<<con->GetPort() <<
+        					" -- stop sending requesting for now");
         			bStopRequesting=true;
         			break;
         		}
-        		else if(inpipe.size() > UINT(acfg::pipelinelen))
-        		{
-        			LOG("Pipeline overrun, stop requesting for now");
-        			break;
-        		}
-        		else
-        		{
-        			cjob->AppendRequest(m_sendBuf, m_sXForwardedFor);
-        			LOG("request added to buffer");
-        			inpipe.push_back(cjob);
-        			m_qNewjobs.pop_front();
 
-        			if(m_nTempPipelineDisable >0)
-        			{
-        				bStopRequesting=true;
-        				--m_nTempPipelineDisable;
-        				break;
-        			}
-        		}
+				cjob->AppendRequest(m_sendBuf, m_sXForwardedFor, m_proxied);
+				LOG("request added to buffer");
+				inpipe.push_back(cjob);
+				m_qNewjobs.pop_front();
+
+				if (m_nTempPipelineDisable > 0)
+				{
+					bStopRequesting = true;
+					--m_nTempPipelineDisable;
+					break;
+				}
+
         	}
         }
 
@@ -1055,21 +1142,16 @@ void dlcon::WorkLoop()
         	return;
         }
 
-        // either have something to receive or something to send (sooner or later)
-		ASSERT( !inpipe.empty() || !bStopRequesting);
-
-		// must test this connection, just be sure no crap is in the pipe
-		if (bUsed && (bUsed = false, check_read_state(con->GetFD())))
-		{
-			ldbg("code: MoonWalker");
-			con.reset();
-			continue;
-		}
-
         // inner loop: plain communication until something happens. Maybe should use epoll here?
         loopRes=ExchangeData(sErrorMsg, con, inpipe);
         ldbg("loopRes: "<< loopRes);
 
+        /* check whether we have a pipeline stall. This may happen because a) we are done or
+         * b) because of the remote hostname change or c) the client stopped sending tasks.
+         * Anyhow, that's a reason to put the connection back into the shared pool so either we
+         * get it back from the pool in the next workloop cycle or someone else gets it and we
+         * get a new connection for the new host later.
+         * */
         if (inpipe.empty())
 		{
         	// all requests have been processed (client done, or pipeline stall, who cares)
@@ -1088,16 +1170,26 @@ void dlcon::WorkLoop()
 			}
 		}
 
+
+        /*
+         * Here we go if the inpipe is still not processed or there have been errors
+         * needing special handling.
+         */
+
         if( (HINT_DISCON|EFLAG_LOST_CON) & loopRes)
         {
         	dbgline;
         	con.reset();
+        	m_inBuf.clear();
+        	m_sendBuf.clear();
         }
 
         if ( loopRes & HINT_TGTCHANGE )
         {
-        	// short queue has been rewritten, needs to play back everything
-        	// if not reset above -> should be in good shape
+        	// short queue continues jobs with rewritten targets, so
+        	// reinsert them into the new task list and continue
+
+        	// if conn was not reset above then it should be in good shape
         	tcpconnect::RecycleIdleConnection(con);
         	goto move_jobs_back_to_q;
         }
