@@ -535,7 +535,7 @@ void expiration::Action(const string & cmd)
 	m_damageList.open(SZABSPATH(FNAME_DAMAGED), ios::out | ios::trunc);
 
 	SendChunk("<b>Validating cache contents...</b><br>\n");
-	_ProcessSeenIndexFiles();
+	ProcessSeenIndexFiles(*this);
 
 	if(CheckAndReportError() || CheckStopSignal())
 		goto save_fail_count;
