@@ -34,29 +34,6 @@ string fileitem::GetHttpMsg()
 	return m_head.frontLine;
 }
 
-/*
-fileitem_base::fileitem_base(MYSTD::string sPath) :
-	condition(),
-	m_nIncommingCount(0),
-	m_nSizeSeen(0),
-	m_bCheckFreshness(true),
-	m_bHeadOnly(false),
-	m_nRangeLimit(-1),
-	m_bAllowStoreData(true),
-	m_nSizeChecked(0),
-	m_filefd(-1),
-	m_nDlRefsCount(0),
-	m_status(FIST_FRESH),
-	m_nTimeDlDone(END_OF_TIME)
-{
-
-	m_sPathRel = sPath;
-	m_sPathAbs = acfg::cacheDirSlash+sPath;
-
-	//ldbg("item created, m_sPath: "<< m_sPath);
-}
-*/
-
 fileitem::fileitem() :
 	condition(),
 	m_nIncommingCount(0),
@@ -303,7 +280,7 @@ fileitem::FiStatus fileitem::WaitForFinish(int *httpCode)
 
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
-	aclog::errnoFmter f;
+	errnoFmter f;
 	aclog::err(tSS() << sFile <<
 			" storage error [" << msg << "], last errno: " << f);
 }
@@ -561,7 +538,7 @@ bool fileitem_with_storage::DownloadStartedStoreHeader(const header & h, const c
 		
 		if (m_filefd<0)
 		{
-			aclog::errnoFmter efmt("503 Cache storage error - ");
+			errnoFmter efmt("503 Cache storage error - ");
 #ifdef DEBUG
 			BOUNCE((efmt+sPathAbs).c_str());
 #else
@@ -677,7 +654,7 @@ bool fileitem_with_storage::StoreFileData(const char *data, unsigned int size)
 				{
 					if(EINTR==errno || EAGAIN==errno)
 						continue;
-					aclog::errnoFmter efmt("HTTP/1.1 503 ");
+					errnoFmter efmt("HTTP/1.1 503 ");
 					m_head.frontLine = efmt;
 					m_status=FIST_DLERROR;
 					// message will be set by the caller
