@@ -12,13 +12,12 @@
 #include "sha1.h"
 #include "csmapping.h"
 
-#define FLAG_EOF 8
-#define FLAG_ERROR 16
-#define FLAG_COMPEOF 32
-
 #include <iostream>
 
-#define BUFSIZE 20000
+// must be something sensible, ratio impacts stack size by inverse power of 2
+#define BUFSIZEMIN 4095 // makes one page on i386 and should be enough for typical index files
+#define BUFSIZEMAX 16*4096
+
 
 #ifdef MINIBUILD
 #undef HAVE_LIBBZ2
@@ -32,12 +31,10 @@ using namespace MYSTD;
 class IDecompressor
 {
 public:
-	bool inited;
-	IDecompressor() : inited(false) {};
+	bool inited, eof;
+	IDecompressor() : inited(false), eof(false) {};
 	virtual ~IDecompressor() {};
-	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
-	{ return FLAG_ERROR; }
-
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf) =0;
 };
 
 #ifdef HAVE_LIBBZ2
@@ -56,23 +53,25 @@ public:
 		BZ2_bzDecompressEnd(&strm);
 		inited = false;
 	}
-	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
 	{
-		strm.next_in=szInBuf+nBufPos;
-		strm.avail_in=nBufSize-nBufPos;
-		strm.next_out=UncompBuf.wptr();
-		strm.avail_out=UncompBuf.freecapa();
+		strm.next_in = szInBuf + nBufPos;
+		strm.avail_in = nBufSize - nBufPos;
+		strm.next_out = UncompBuf.wptr();
+		strm.avail_out = UncompBuf.freecapa();
 
-	int ret=BZ2_bzDecompress(&strm);
-	if(ret==BZ_STREAM_END || ret==BZ_OK)
-	{
-		nBufPos += ( (nBufSize-nBufPos) - strm.avail_in);
-		unsigned int nGotBytes= UncompBuf.freecapa() - strm.avail_out;
-		UncompBuf.got(nGotBytes);
-		return ret == BZ_STREAM_END ? FLAG_COMPEOF : 0;
-	}
-	// or corrupted data?
-	return (FLAG_COMPEOF|FLAG_ERROR);
+		int ret = BZ2_bzDecompress(&strm);
+		if (ret == BZ_STREAM_END || ret == BZ_OK)
+		{
+			nBufPos = nBufSize - strm.avail_in;
+			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			UncompBuf.got(nGotBytes);
+			eof = ret == BZ_STREAM_END;
+			return true;
+		}
+		// or corrupted data?
+		eof = true;
+		return false;
 	}
 };
 #else
@@ -95,7 +94,7 @@ public:
 		deflateEnd(&strm);
 		inited = false;
 	}
-	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
 	{
 		strm.next_in = (uint8_t*) szInBuf + nBufPos;
 		strm.avail_in = nBufSize - nBufPos;
@@ -105,13 +104,15 @@ public:
 		int ret = inflate(&strm, Z_NO_FLUSH);
 		if (ret == Z_STREAM_END || ret == Z_OK)
 		{
-			nBufPos += ((nBufSize - nBufPos) - strm.avail_in);
-			unsigned int nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			nBufPos = nBufSize - strm.avail_in;
+			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
 			UncompBuf.got(nGotBytes);
-			return ret == Z_STREAM_END ? FLAG_COMPEOF : 0;
+			eof = ret == Z_STREAM_END;
+			return true;
 		}
 		// or corrupted data?
-		return (FLAG_COMPEOF | FLAG_ERROR);
+		eof = true;
+		return false;
 	}
 };
 
@@ -142,23 +143,25 @@ public:
 		lzma_end(&strm);
 		inited = false;
 	}
-	virtual UINT UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
+	virtual bool UncompMore(char *szInBuf, size_t nBufSize, size_t &nBufPos, acbuf &UncompBuf)
 	{
 		strm.next_in = (uint8_t*) szInBuf + nBufPos;
 		strm.avail_in = nBufSize - nBufPos;
 		strm.next_out = (uint8_t*) UncompBuf.wptr();
 		strm.avail_out = UncompBuf.freecapa();
 
-		lzma_ret ret=lzma_code(&strm, LZMA_RUN);
+		lzma_ret ret=lzma_code(&strm, strm.avail_in ? LZMA_RUN : LZMA_FINISH);
 		if (ret == LZMA_STREAM_END || ret == LZMA_OK)
 		{
-			nBufPos += ((nBufSize - nBufPos) - strm.avail_in);
-			unsigned int nGotBytes = UncompBuf.freecapa() - strm.avail_out;
+			nBufPos = nBufSize - strm.avail_in;
+			UINT nGotBytes = UncompBuf.freecapa() - strm.avail_out;
 			UncompBuf.got(nGotBytes);
-			return ret == LZMA_STREAM_END ? FLAG_COMPEOF : 0;
+			eof = ret == LZMA_STREAM_END;
+			return true;
 		}
 		// or corrupted data?
-		return (FLAG_COMPEOF | FLAG_ERROR);
+		eof = true;
+		return false;
 	}
 };
 #else
@@ -172,7 +175,8 @@ public:
 
 filereader::filereader() 
 :
-	flags(FLAG_ERROR|FLAG_EOF),
+	m_bError(false),
+	m_bEof(false),
 	m_szFileBuf((char*)MAP_FAILED),
 	m_nBufSize(0),
 	m_nBufPos(0),
@@ -204,7 +208,7 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	else if (endsWithSzAr(sFilename, ".gz"))
 		m_Dec.reset(new tGzDec);
 	else if(endsWithSzAr(sFilename, ".xz"))
-		m_Dec.reset(new tXzDec);
+		m_Dec.reset(new tXzDec(false));
 	else if (endsWithSzAr(sFilename, ".lzma"))
 		m_Dec.reset(new tXzDec(true));
 	else // unknown... ok, probe it
@@ -230,7 +234,8 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 			//aclog::err("Unable to uncompress file, algo not available");
 			return false;
 		}
-		m_UncompBuf.init(BUFSIZE);
+		m_UncompBuf.clear();
+		m_UncompBuf.setsize(BUFSIZEMIN);
 	}
 
 	struct stat statbuf;
@@ -251,6 +256,11 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 			return false;
 		m_nBufSize = statbuf.st_size;
 	}
+	else if(m_Dec.get())
+	{
+		// compressed file but empty -> error
+		return false;
+	}
 	else
 	{
 		m_szFileBuf = NULL;
@@ -264,25 +274,22 @@ bool filereader::OpenFile(const string & sFilename, bool bNoMagic)
 	
 	m_nBufPos=0;
 	m_nCurLine=0;
-	flags = 0;
+	m_bError = m_bEof = false;
 	return true;
 }
 
 bool filereader::CheckGoodState(bool bErrorsConsiderFatal, cmstring *reportFilePath) const
-{	
-	if (flags&FLAG_ERROR)
-	{
-		if(bErrorsConsiderFatal)
-		{
-			cerr << "Error opening file";
-			if(reportFilePath)
-				cerr << " " << *reportFilePath;
-			cerr << ", terminating."<<endl;
-			exit(EXIT_FAILURE);
-		}
+{
+	if (!m_bError)
+		return true;
+	if (!bErrorsConsiderFatal)
 		return false;
-	}
-	return true;
+	cerr << "Error opening file";
+	if (reportFilePath)
+		cerr << " " << *reportFilePath;
+	cerr << ", terminating." << endl;
+	exit(EXIT_FAILURE);
+
 }
 
 void filereader::Close()
@@ -300,56 +307,61 @@ void filereader::Close()
 
 	m_nBufSize=0;
 
-	flags = (FLAG_ERROR|FLAG_EOF); // will be cleared in open method
+	m_bError = m_bEof = true; // will be cleared in Open()
 }
 
 filereader::~filereader() {
 	Close();
 }
 
-// TODO: can split up a line when it doesn't fit into one buffer
-bool filereader::GetOneLine(string & sOut) {
+bool filereader::GetOneLine(string & sOut, bool bForceUncompress) {
 	
 	sOut.clear();
 	
+	if(m_bEof && m_nEofLines-- >0)
+		return true; // just the empty line
+
 	// stop flags set in previous run
-	if(flags & (FLAG_EOF|FLAG_ERROR))
+	if(m_bError || m_bEof)
 		return false;
 	
-	//cout<< "wtf, m_pos: " << m_pos <<endl;
-	
 	const char *rbuf;
 	size_t nRest;
-	bool bCanRetry=true;
 
-	maybe_got_more:
-	
-	if(!m_Dec.get())
+	// navigate to the place where the reading starts
+
+	// needing an uncompression step? Either in the beginning or when doing the subcall
+	if(m_Dec.get())
 	{
-		bCanRetry=false;
-		
-		if(m_nBufPos>=m_nBufSize)
-			flags|=FLAG_EOF;
-		// detect eof and remember that, for now or later calls
-		nRest = (flags&FLAG_EOF) ? 0 : m_nBufSize-m_nBufPos;
-		rbuf=m_szFileBuf+m_nBufPos;
-	}
-	else 
-	{
-		nRest=m_UncompBuf.size();
-		
-		if(nRest==0 && (flags&(FLAG_COMPEOF|FLAG_ERROR|FLAG_EOF)))
+		if ((!m_nBufPos || bForceUncompress) && !m_Dec->eof)
 		{
-			if(m_nEofLines-- >0)
+			m_UncompBuf.move(); // make free as much as possible
+
+			// must uncompress but the buffer is full. Resize if, it not possible, fail only then.
+			if(bForceUncompress && 0==m_UncompBuf.freecapa())
 			{
-				sOut.clear();
-				return true;
+				if (m_UncompBuf.totalcapa() >= BUFSIZEMAX
+						|| !m_UncompBuf.setsize(m_UncompBuf.totalcapa() * 2))
+				{
+					m_bError = true;
+					return false;
+				}
 			}
-			return false;
+
+			m_bError = ! m_Dec->UncompMore(m_szFileBuf, m_nBufSize, m_nBufPos, m_UncompBuf);
 		}
-		
+
+		nRest=m_UncompBuf.size();
 		rbuf=m_UncompBuf.rptr();
 	}
+	else
+	{
+		if(m_nBufPos>=m_nBufSize)
+			m_bEof=true;
+		// detect eof and remember that, for now or later calls
+		nRest = m_bEof ? 0 : m_nBufSize-m_nBufPos;
+		rbuf=m_szFileBuf+m_nBufPos;
+	}
 	
 	// look for end in the rest of buffer (may even be nullsized then it fails implicitely, newline decides), 
 	// on miss -> try to get more, check whether the available size changed, 
@@ -381,13 +393,15 @@ bool filereader::GetOneLine(string & sOut) {
 	}
 	else
 	{
-		if(bCanRetry)
+		if(m_Dec.get())
 		{
-			bCanRetry=false;
-			UncompressMoreData();
-			goto maybe_got_more;
+			// it's the final buffer...
+			if(m_Dec->eof)
+				m_bEof=true; // won't uncompress more anyhow
+			else // otherwise has to unpack more with a larger buffer or fail
+				return GetOneLine(sOut, true);
 		}
-		
+
 		// otherwise can continue to the finish 
 		nDropLen=nLineLen=nRest;
 	}
@@ -403,87 +417,6 @@ bool filereader::GetOneLine(string & sOut) {
 	return true;
 }
 
-//! @return: new text buffer size
-inline void filereader::UncompressMoreData() {
-
-	// work with uncompressed buffer/window...
-	m_UncompBuf.move(); // get unused space if possible
-		
-	if(	(flags&FLAG_COMPEOF) // cannot uncompress more
-		|| m_UncompBuf.freecapa()==0 )
-		return;
-	
-	if(m_nBufPos>=m_nBufSize )
-	{
-		// shouldn't be here. Decompressor's errors or eof must have been handled before. Undiscovered somehow?
-		flags|=(FLAG_ERROR|FLAG_EOF);
-		return;
-	}
-	
-	UINT wtf=m_Dec->UncompMore(m_szFileBuf, m_nBufSize, m_nBufPos, m_UncompBuf);
-	flags |= wtf;
-
-}
-
-/*
-// TODO: make this use a string?
-bool filereader::GetMd5String(const MYSTD::string & sFileName, char out[])
-{
-	uint8_t buf[16];
-	if(!GetMd5Sum(sFileName, buf))
-		return false;
-	
-	for(UINT i=0;i<16;i++)
-		sprintf(&out[2*i], "%02x", buf[i]);
-	
-	return true;
-}
-*/
-/*
-bool filereader::GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack,
-		off_t &scannedSize)
-{
-	md5_state_s ctx;
-	md5_init(&ctx);
-	filereader f;
-	scannedSize=0;
-	if (!f.OpenFile(sFileName, !bTryUnpack))
-		return false;
-	if (f.flags&FLAG_PLAIN)
-	{
-		md5_append(&ctx, (md5_byte_t*) f.m_szFileBuf, f.m_nBufSize);
-		scannedSize=f.m_nBufSize;
-	}
-	else
-	{
-		while(true)
-		{
-			f._UncompressMoreData();
-			if(f.flags&FLAG_ERROR)
-				return false;
-			UINT nRest=f.m_UncompBuf.size();
-			if(nRest==0)
-				break;
-			md5_append(&ctx, (md5_byte_t*) f.m_UncompBuf.rptr(), nRest);
-			scannedSize+=nRest;
-			f.m_UncompBuf.clear();
-		}
-		
-	}
-	
-	md5_finish(&ctx, out);
-	return f.CheckGoodState(false);
-}
-
-bool filereader::GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], 
-		bool bTryUnpack, off_t &scannedSize, FILE *fDump)
-{
-	filereader f;
-	return (f.OpenFile(sFileName, !bTryUnpack)
-			&& f.GetSha1Sum(out, scannedSize, fDump));
-}
-*/
-
 #ifndef MINIBUILD
 class csumSHA1 : public csumBase, public SHA_INFO
 {
@@ -536,20 +469,27 @@ bool filereader::GetChecksum(int csType, uint8_t out[], off_t &scannedSize, FILE
 	}
 	else
 	{
-		while (true)
+		for(;;)
 		{
-			UncompressMoreData();
-			if (flags&FLAG_ERROR)
+
+			if(!m_Dec->UncompMore(m_szFileBuf, m_nBufSize, m_nBufPos, m_UncompBuf))
+			{
+				m_bError=true;
 				return false;
-			UINT nRest=m_UncompBuf.size();
-			if (nRest==0)
-				break;
-			summer->add(m_UncompBuf.rptr(), nRest);
-			//sha_update(&ctx, (md5_byte_t*) m_UncompBuf.rptr(), nRest);
+			}
+
+			UINT nPlainSize=m_UncompBuf.size();
+			summer->add(m_UncompBuf.rptr(), nPlainSize);
 			if(fDump)
-				fwrite(m_UncompBuf.rptr(), sizeof(char), nRest, fDump);
-			scannedSize+=nRest;
+				fwrite(m_UncompBuf.rptr(), sizeof(char), nPlainSize, fDump);
+			scannedSize+=nPlainSize;
 			m_UncompBuf.clear();
+
+			if(m_Dec->eof)
+			{
+				m_bEof=true;
+				break;
+			}
 		}
 
 	}
@@ -584,34 +524,6 @@ void check_algos()
 }
 
 
-/*
-foreach $b (0..255) {
-   print "\n" if($b%16==0);
-   if( $b>=48 && $b<58 ) { $b-=48;}
-   elsif($b>=97 && $b<103) { $b-=87;}
-   elsif($b>=65 && $b<71) { $b-=55;}
-   else {$b= --$dummy}
-   print "$b,";
-}
-print "\n";
-*/
-
-
-/*
-bool filereader::CheckMd5Sum(const string & sFileName, const string & sReference)
-{
-	uint8_t sum[16];
-
-	if(sFileName.length()!=16)
-		return false;
-	
-	if(!GetMd5Sum(sFileName, sum))
-		return false;
-	
-	return CsEqual( (unsigned char*) sReference.data(), sum, 16);
-}
-
-*/
 
 #endif
 
@@ -654,17 +566,3 @@ bool Bz2compressFile(const char *pathIn, const char*pathOut)
 }
 
 #endif
-
-/*
- * Playing (z|bz|xz)cat
-int main(int argc, char **argv)
-{
-	filereader fh;
-	fh.OpenFile(argv[1]);
-	mstring sLine;
-	while(fh.GetOneLine(sLine))
-	{
-		cout << sLine <<endl;
-	}
-}
-*/
