@@ -535,10 +535,11 @@ void job::PrepareDownload() {
 
 		char *pEnd(0);
 		UINT nPort = 80;
-		if(!tUrl.sPort.empty())
+		LPCSTR sPort=tUrl.GetPort().c_str();
+		if(!*sPort)
 		{
-			nPort = (UINT) strtoul(tUrl.sPort.c_str(), &pEnd, 10);
-			if('\0' != *pEnd || pEnd == tUrl.sPort.c_str() || nPort > TCP_PORT_MAX || !nPort)
+			nPort = (UINT) strtoul(sPort, &pEnd, 10);
+			if('\0' != *pEnd || pEnd == sPort || nPort > TCP_PORT_MAX || !nPort)
 				goto report_invport;
 		}
 
@@ -1026,7 +1027,7 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 			bool bFreshnessForced = (m_type != rechecks::FILE_INDEX
 				|| m_pReqHead->h[header::ACNGFSMARK] || !pIfmo);
 
-			struct tm tm1, tm2;
+			struct tm tm1={0}, tm2={0};
 			bool bIfModSeenAndChecked=false;
 			if(pIfmo && header::ParseDate(pIfmo, &tm1) && header::ParseDate(pLastMo, &tm2))
 			{
@@ -1062,8 +1063,6 @@ inline const char * job::BuildAndEnqueHeader(const fileitem::FiStatus &fistate,
 
 						)
 					{
-						USRDBG("request range problem: ");
-
 						// detect errors, out-of-range case
 						if(m_nReqRangeFrom>=nContLen || m_nReqRangeTo<m_nReqRangeFrom)
 							return "416 Requested Range Not Satisfiable";
@@ -1163,6 +1162,3 @@ void job::SetErrorResponse(const char * errorLine, const char *szLocation)
 	//aclog::err(tSS() << "fileitem is now " << uintptr_t(m_pItem.get()));
 	m_state=STATE_SEND_MAIN_HEAD;
 }
-
-
-
