@@ -20,6 +20,10 @@ using namespace MYSTD;
 
 cmstring sPathSep(SZPATHSEP);
 cmstring sPathSepUnix(SZPATHSEPUNIX);
+#ifndef MINIBUILD
+cmstring sDefPortHTTP("80");
+cmstring sDefPortHTTPS("443");
+#endif
 
 /*
 int getUUID() {
@@ -34,6 +38,13 @@ void set_nb(int fd) {
     flags |= O_NONBLOCK;
     flags = fcntl(fd, F_SETFL, flags);
 }
+void set_block(int fd) {
+    int flags = fcntl(fd, F_GETFL);
+    //ASSERT(flags != -1);
+    flags &= ~O_NONBLOCK;
+    flags = fcntl(fd, F_SETFL, flags);
+}
+
 
 /*
 inline tStrPos findHostStart(const MYSTD::string & sUri)
@@ -188,6 +199,11 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 	sPort.clear();
 	sHost.clear();
 	sPath.clear();
+	sUserPass.clear();
+#ifdef HAVE_SSL
+	bSSL=false;
+#endif
+	bIsTransferlEncoded=false;
 	
 	mstring url = UrlUnescape(sUrlRaw);
 
@@ -202,10 +218,16 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 	
 	if(0==strncasecmp(url.c_str(), "http://", 7))
 		hStart=7;
-	/* checked below with others
-	else if(url[0]=='/')
-		hStart=1;
-		*/
+	else if(0==strncasecmp(url.c_str(), "https://", 8))
+	{
+#ifndef HAVE_SSL
+	aclog::err("E_NOTIMPLEMENTED: SSL");
+	return false;
+#else
+		hStart=8;
+		bSSL=true;
+#endif
+	}
 	else if(isalnum((UINT)url[0]))
 		hStart=0;
 	else if(url[0]=='[')
@@ -242,6 +264,14 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 	
 	sHost=url.substr(hStart, hEndSuc-hStart);
 
+	// credentials might in there, strip them of
+	l=sHost.rfind('@');
+	if(l!=mstring::npos)
+	{
+		sUserPass=sHost.substr(0, l);
+		sHost.erase(0, l+1);
+	}
+
 	l=sHost.size();
 	p=sHost.rfind(':');
 	if(p==stmiss)
@@ -274,8 +304,13 @@ bool tHttpUrl::SetHttpUrl(cmstring &sUrlRaw)
 
 string tHttpUrl::ToURI(bool bUrlEscaped) const
 {
-	string s("http://");
-	if (bUrlEscaped)
+#ifdef HAVE_SSL
+	mstring s(bSSL ? "https://" : "http://");
+#else
+	mstring s("http://");
+#endif
+	// if needs transfer escaping and is not internally escaped
+	if (bUrlEscaped && !bIsTransferlEncoded)
 	{
 		UrlEscapeAppend(sHost, s);
 		if (!sPort.empty())
@@ -375,6 +410,18 @@ int GetSimilarity(cmstring& wanted, cmstring& candidate)
 #endif
 
 
+/*
+foreach $b (0..255) {
+   print "\n" if($b%16==0);
+   if( $b>=48 && $b<58 ) { $b-=48;}
+   elsif($b>=97 && $b<103) { $b-=87;}
+   elsif($b>=65 && $b<71) { $b-=55;}
+   else {$b= --$dummy}
+   print "$b,";
+}
+print "\n";
+*/
+
 #define _inv MAX_VAL(uint_fast16_t)
 
 uint_fast16_t hexmap[] = {
@@ -561,3 +608,55 @@ bool UrlUnescapeAppend(cmstring &from, mstring & to)
 	return ret;
 }
 
+
+string EncodeBase64Auth(cmstring & s)
+{
+	int cols=0, bits=0, c=0, char_count=0;
+	char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+	tStrPos pos=0;
+	string out;
+	while ( pos<s.size() )
+	{
+		c=s[pos++];
+		if('%' == c && pos<s.length()-1
+				&& hexmap[(unsigned)s[pos]]<=15
+				&& hexmap[(unsigned)s[pos+1]]<=15)
+		{
+			c=char(16*hexmap[(unsigned)s[pos]]+hexmap[(unsigned)s[pos+1]]);
+			pos+=2;
+		}
+		bits += c;
+		char_count++;
+		if (char_count == 3)
+		{
+			out+=(alphabet[bits >> 18]);
+			out+=(alphabet[(bits >> 12) & 0x3f]);
+			out+=(alphabet[(bits >> 6) & 0x3f]);
+			out+=(alphabet[bits & 0x3f]);
+			cols += 4;
+			bits = 0;
+			char_count = 0;
+		}
+		else
+		{
+			bits <<= 8;
+		}
+	}
+	if (char_count != 0)
+	{
+		bits <<= 16 - (8 * char_count);
+		out+=(alphabet[bits >> 18]);
+		out+=(alphabet[(bits >> 12) & 0x3f]);
+		if (char_count == 1)
+		{
+			out+=('=');
+			out+=('=');
+		}
+		else
+		{
+			out+=(alphabet[(bits >> 6) & 0x3f]);
+			out+=('=');
+		}
+	}
+	return out;
+}
